#!/usr/bin/env python 
"""
slicer.py
Version: 0.1.1 (bugfix release)
Author: Jari Pennanen 'Ciantic', 2009
Website: http://ciantic.blogspot.com/2009/02/inkscape-slicer.html 
CPL (Common Public License)

Usage: 
1.) Create layer e.g. named 'slices'. 
2.) Create rects as your slices, set their Label (select rect and Object -> Object properties) as the output filename (without extension). 
3.) Run Effects -> Export -> Slicer, specify directory where you want your slices to be saved and Apply.

See http://ciantic.blogspot.com/2009/02/inkscape-slicer.html for more info.


Original idea from Matt Harrison:
http://panela.blog-city.com/slicing_web_pages_with_inkscape_and_python.htm
WARNING, Matt's version didn't work with latest Ubuntu (8.10) and Inkscape 0.46, 
that was the reason I rewrote whole script with features I needed. I have no intention
to ripoff Matt's effort, so I fully thank Matt for his work, and if you somehow feel that
I have broken the license in Matt's file you can contact me through blog url above.

For good pixel exports set the Document Properties default units to "px".
"""
import os
import sys
import subprocess
from xml.etree import ElementTree

sys.path.append('/usr/share/inkscape/extensions')

import inkex
import simplestyle

# TODO: Split to logical methods: export, ...
# TODO: Create ability to trim images: By trimming I mean leave only visible part behind.
# Perhaps first doing this and then manipulating output png by removing empty?
# If there were a Inscapish way to do it...

# For debugger buds: this is not Win32 friendly filepath, but it can be changed by debug platform ;)
#~ import logging
#~ logging.basicConfig(filename="/tmp/inklog.log", level=logging.DEBUG)

class ExportSlices(inkex.Effect):
    """Exports all rectangles in the given layer"""
    def __init__(self):
        inkex.Effect.__init__(self)
        
        self.OptionParser.add_option("-d", "--directory",
                                     action="store", type="string", 
                                     dest="directory", default=os.path.expanduser("~"),
                                     help="Existing destination directory")
        self.OptionParser.add_option("-l", "--layer",
                                     action="store", type="string",
                                     dest="layer", default="slices",
                                     help="Layer with slices (rects) in it")
        self.OptionParser.add_option("-o", "--overwrite",
                                     action="store", type="inkbool",
                                     dest="overwrite", default=False,
                                     help="Overwrite existing files")
        
    def effect(self):
        """
        self.args[-1] contains a copy of the current document.  If you want
        to update it before exporting, update that copy.
        
        Do NOT use print statements inside these files, it will only break the parsing. 
        
        Instead do something like this:
        logging.basicConfig(filename="/tmp/inklog.log", level=logging.DEBUG)
        Open new term with: "tail -f /tmp/inklog.log"
        And in code: logging.debug('my debug!')
        """
        temp_filepath = self.args[-1]
        imageRoot = self.document.getroot()
        
        #~ logging.debug(temp_filepath)
        #~ logging.debug(self.document)
        #~ logging.debug(ElementTree.tostring(imageRoot))
        #~ logging.debug(self.options.layer)
        #~ logging.debug(ElementTree.tostring(self.document.getroot()))
        
        sliceLayer = None
        # For each group (layer):
        for group in imageRoot.findall("{http://www.w3.org/2000/svg}g"): # TODO: This: {http://www.w3.org/2000/svg}g[@{http://www.inkscape.org/namespaces/inkscape}label='slices'] didn't work, why?
            #~ logging.debug(group)
            #~ logging.debug(group.attrib)
            
            # If group's label attribute is wanted, then we found the layer
            try:
                if group.attrib['{http://www.inkscape.org/namespaces/inkscape}label'] == self.options.layer:
                    sliceLayer = group
                    break
            except KeyError, ker:
                pass
        
        # If slice layer was not found:
        if sliceLayer is None:
            sys.stderr.write('Layer named \'%s\' was not found!' % self.options.layer)
            sys.exit(1)
            
        # Get slicing rectangles from sliceLayer
        slicing_rects = [rect for rect in sliceLayer.findall('{http://www.w3.org/2000/svg}rect')]
        
        if len(slicing_rects) == 0:
            sys.stderr.write('There are no rects in layer named \'%s\'!\n' % self.options.layer)
            sys.exit(1)
        
        backup_styles = []
        
        # Set the style of all slicing rects to export state
        for slicing_rect in slicing_rects:
            # Backup old style
            backup_styles.append(slicing_rect.attrib['style'])
            
            # Our export state currently is just setting opacity to zero, and stroke to none
            styles = simplestyle.parseStyle(slicing_rect.attrib['style'])
            styles['opacity'] = '0.0'
            styles['stroke'] = 'none'
            slicing_rect.attrib['style'] = simplestyle.formatStyle(styles)
        
        self.document.write(temp_filepath)
            
        # For each slice do the slicing and save to appropriate pngs:
        for slicing_rect in slicing_rects:
            rect_id = slicing_rect.attrib['id']
            
            # Fallback to id as filename if label is not given (TODO: This is unspecified hidden feature!)
            try:
                rect_label = slicing_rect.attrib['{http://www.inkscape.org/namespaces/inkscape}label']
            except KeyError, ker:
                basename = rect_id
            else:
                basename = rect_label.lstrip("#")
            filename = "%s.png" % basename
            
            output_filepath = os.path.join(os.path.abspath(self.options.directory), filename)
            # TODO: Validate filename and directory
            
            if os.path.exists(output_filepath) and not self.options.overwrite:
                sys.stderr.write('File \'%s\' exists already, you must set to overwrite!\n' % output_filepath)
                continue
            
            # TODO: inkscape in windows is not in path
            command = ["inkscape", "-i", rect_id, "-e", output_filepath, temp_filepath]
            
            p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            stdout = p.communicate()[0]
            retcode = p.wait() # This ensures the same thread, without this the reverting might happen too early
            
            #~ logging.debug('executing: %s' % command)
            #~ logging.debug('stdout: %s' % stdout)
            #~ logging.debug('retcode: %s' % retcode)
            
        # Revert the style values back
        for slicing_rect, backup_style in zip(slicing_rects, backup_styles):
            slicing_rect.attrib['style'] = backup_style
        
        self.document.write(temp_filepath)
        
        #sys.stdout.write('All done')
        #sys.exit(0)
        

def _main():
    e = ExportSlices()
    e.affect()
    
if __name__ == "__main__":
    _main()
